@@ -1,7 +1,7 @@
 {-# LANGUAGE OverloadedStrings #-}
 {-|
 Module      : HsLua.Packaging.UDType
-Copyright   : © 2020-2024 Albert Krewinkel
+Copyright   : © 2020-2026 Albert Krewinkel
 License     : MIT
 Maintainer  : Albert Krewinkel <tarleb@hslua.org>
 
@@ -31,7 +31,6 @@ module HsLua.Packaging.UDType
   , initType  -- Reexported from ObjectOrientation
   , udparam
   , udresult
-  , udDocs
   , udTypeSpec
     -- * Helper types for building
   , Member
@@ -47,12 +46,10 @@ import HsLua.Marshalling
 import HsLua.ObjectOrientation
 import HsLua.ObjectOrientation.Operation (metamethodName)
 import HsLua.Packaging.Function
+import HsLua.Packaging.Types (DocumentedType, setName)
 import HsLua.Typing (pushTypeSpec)
 import qualified Data.Map as Map
 
--- | Type definitions containing documented functions.
-type DocumentedType e a = UDType e (DocumentedFunction e) a
-
 -- | A userdata type, capturing the behavior of Lua objects that wrap
 -- Haskell values. The type name must be unique; once the type has been
 -- used to push or retrieve a value, the behavior can no longer be
@@ -92,7 +89,7 @@ method f = methodGeneric (functionName f) f
 operation :: Operation             -- ^ the kind of operation
           -> DocumentedFunction e  -- ^ function used to perform the operation
           -> (Operation, DocumentedFunction e)
-operation op f = (,) op $ setName (metamethodName op) f
+operation op f = (,) op $ f `setName` metamethodName op
 
 -- | Defines a function parameter that takes the given type.
 udparam :: LuaError e
