@@ -1,8 +1,8 @@
 {-|
 Module      : HsLua.PackagingTests
-Copyright   : © 2020-2022 Albert Krewinkel
+Copyright   : © 2020-2026 Albert Krewinkel
 License     : MIT
-Maintainer  : Albert Krewinkel <tarleb+hslua@zeitkraut.de>
+Maintainer  : Albert Krewinkel <tarleb@hslua.org>
 
 Test packaging
 -}
@@ -12,7 +12,6 @@ import Test.Tasty (TestTree, testGroup)
 import qualified HsLua.Packaging.DocumentationTests
 import qualified HsLua.Packaging.FunctionTests
 import qualified HsLua.Packaging.ModuleTests
-import qualified HsLua.Packaging.RenderingTests
 import qualified HsLua.Packaging.UDTypeTests
 
 -- | Tests for package creation.
@@ -20,7 +19,6 @@ tests :: TestTree
 tests = testGroup "Packaging"
   [ HsLua.Packaging.FunctionTests.tests
   , HsLua.Packaging.ModuleTests.tests
-  , HsLua.Packaging.RenderingTests.tests
   , HsLua.Packaging.UDTypeTests.tests
   , HsLua.Packaging.DocumentationTests.tests
   ]
